# frozen_string_literal: true

module Hermes
  # HousingApplication serializer
  class HousingApplicationSerializer < Hermes::ApplicationSerializer
    #cache key: proc { "#{object.cache_key}" }, expires_in: 1.day
    attributes %i[
      applicated_at
      applicant_id
      housing_request_id
      housing_offer_id
      max_date
      epci
      effort_rate
      profile_epci_score
      status
      status_updated_at
      visit_status
      visit_status_updated_at
      score_with_position
      eligible_position
      applicated_position
      effort_rate
      score_id
      housing_request
      visit_conducted_date
      unique_numbers
      
      offer_external_ref
      main_applicant_first_name
      main_applicant_last_name
      request_als_id
      last_status_update_date
      lessor_event_informations
      rank
      last_xml_validation_report
      last_send_has_error
     
    ]
    
    def offer_external_ref
      object.housing_offer.external_ref
    end
    
    def main_applicant_first_name
      object.housing_request.main_applicant.first_name
    end
    
    def main_applicant_last_name
      object.housing_request.main_applicant.last_name
    end
    
    def request_als_id
      object.housing_request.als_id
    end
    
    def unique_numbers
      res = []
      object.housing_request&.nuds&.each do |nud|
        res.push(nud.unique_number)
      end
      res
    end
    
    def embedded_applicant?
      instance_options[:embedded_applicant] == true
    end
    
    attribute :applicant, if: :embedded_applicant?
    
    type 'housing_applications'
    
    def applicant_id
      object.applicant_id&.to_s
    end
    
    def applicant
      return unless object.applicant
      
      Hermes::ApplicantSerializer.new(
        object.applicant,
        scope: scope,
        root: false
      )
    end
    
    def housing_request_id
      object.housing_request_id&.to_s
    end
    
    def housing_offer_id
      object.housing_offer_id&.to_s
    end
    
    def score
      if !object.score_id
        @score ||= Hermes::Score.find_by(
        applicant_id: object.housing_request&.main_applicant_id,
        housing_offer_id: housing_offer_id
      )
      end
    end
    
    def profile_epci_score
      object.profile_epci_score || score&.profile_epci_score
    end
    
    def score_with_position
      object.score_with_position || score&.score_with_position
    end
    
    def eligible_position
      object.eligible_position || score&.eligible_position
    end

    def applicated_position
      object.applicated_position || score&.applicated_position
    end

    def effort_rate
      object.effort_rate || score&.effort_rate
    end
    
    def score_id
      object.score_id || score&.id.to_s
    end
    
    def housing_request
      return unless instance_options[:join_housing_request]
      
      Hermes::HousingRequestSerializer.new(
        object.housing_request,
        scope: scope,
        root: false
      )
    end
  end
end
